#include "node.h"
#include <algorithm>
#include <iostream>
using namespace std;

Node::Node(const Action* action){
	_action = action;
	_preconditions = _action->preconditions();
	_executed = false;
}
Node::Node(){
	_executed = false;
}

Node::Node(const Node& node){
	_action = node._action;
	_preconditions = node._preconditions;
	_executed = node._executed;
}

const Action* Node::action() const{
	return _action;
}


void Node::addChild(Node* node){
	_children.push_back(node);
}

bool Node::executed() const{
	return _executed;
}

const Literals& Node::effects() const{
	return _effects;
}

bool Node::satisfyCondition(std::string effect){
	_preconditions.erase(_preconditions.find(effect));
	return _preconditions.size() == 0 && !_executed;
}

bool Node::satisfyConditions(const Literals& effects){
	for (Literals::const_iterator effect = effects.begin(); effect != effects.end(); ++effect){
		_preconditions.erase(_preconditions.find(*effect));
	}
	return _preconditions.size() == 0 && !_executed;
}

void Node::execute(){
	if(_executed)
		cerr << "Already executed " << _action->name() << endl;
	if ((_preconditions.size() != 0) || _executed)
		return;

	_executed = true;
	int value = _action->execute();
	_effects = _action->effects(value);

}

const std::vector<Node*>& Node::children() const{
	return _children;
}
StartNode::StartNode(const Literals& init){
	EffectsMap initial;
	initial[0] = init;
	_action = new Action("start",Preconditions(),"", initial);
	_executed = true;
	_effects = init;
}

const Preconditions& Node::preconditions() const{
	return _preconditions;
}

EndNode::EndNode(const Literals& goal){
	Preconditions goalState;
	for(Literals::const_iterator g = goal.begin(); g != goal.end(); ++g)
		goalState[*g] = false;
	_action = new Action("finish",goalState,"",EffectsMap());
	_preconditions = _action->preconditions();
}

